from shukujitsu.exports import CountryHoliday
from shukujitsu.exports import Japan


__version__ = "2021.2.1.post2"
# aliases
JP = JPN = Japan
assert __version__ and CountryHoliday and Japan and JP and JPN
